#!/bin/sh

SECONDARY_WAN_CONF="/usr/syno/etc/secondarywan.conf"
SZF_IFCFG_PREFIX="/etc/sysconfig/network-scripts/ifcfg-"
SZF_RDISC6="/usr/bin/rdisc6"
SYNONETD_TOOL="/usr/syno/sbin/synonetdtool"
SYNONET="/usr/syno/sbin/synonet"

# check if this interface need to reload ipv6 dhcp
# if yes, use rdisc6 to reload ipv6 dhcp
check_and_reload_ipv6_dhcp()
{
	local ifname=$1
	local configfile="${SZF_IFCFG_PREFIX}${ifname}"

	# file not exist, or IFNAME is empty
	if [ ! -f ${configfile} ]; then
		return
	fi

	local ipv6init=`/bin/get_key_value ${configfile} IPV6INIT`
	if [ "dhcp" != "${ipv6init}" -a "auto_dhcp" != "${ipv6init}" -a "auto" != "${ipv6init}" ]; then
		return
	fi

	# true, do reload dhcp
	${SYNONETD_TOOL} --add-gateway-info -6 -2 ${IFNAME} NULL NULL ethernet
	$SZF_RDISC6 -n ${ifname} &> /dev/null
}

add_static_route_v4() {
	local ipaddr=`/bin/get_key_value /etc/sysconfig/network-scripts/ifcfg-${IFNAME} IPADDR`
	local netmask=`/bin/get_key_value /etc/sysconfig/network-scripts/ifcfg-${IFNAME} NETMASK`
	local gateway_database="/etc/iproute2/config/gateway_database"
	local gatewayip=''
	local dnsip=''

	/sbin/ifconfig ${IFNAME} 0.0.0.0
	/sbin/ifconfig ${IFNAME} ${ipaddr} netmask ${netmask}

	gatewayip=`/usr/syno/bin/get_section_key_value ${gateway_database} ${IFNAME} gateway`
	if [ -z "$gatewayip" ]; then
		gatewayip="NULL"
	fi
	dnsip=`/usr/syno/bin/get_section_key_value ${gateway_database} ${IFNAME} dns`
	if [ -z "$dnsip" ]; then
		dnsip="NULL"
	fi

	${SYNONETD_TOOL} --add-gateway-info -4 -2 ${IFNAME} ${gatewayip} ${dnsip} ethernet
}

add_static_gateway_v6() {
	local GATEWAY="NULL"
	local DNS="NULL"
	local IFCFG_FILE="/etc/sysconfig/network-scripts/ifcfg-${IFNAME}"
	if [ ! -f ${IFCFG_FILE} ]; then
		return
	fi
	local IPV6INIT=`get_key_value ${IFCFG_FILE} IPV6INIT`
	if [ "${IPV6INIT}" = "off" ]; then
		return
	fi

	local BRIDGE=`get_key_value ${IFCFG_FILE} BRIDGE`

	if [ "${IPV6INIT}" = "static" -a -z "${BRIDGE}" ]; then
		GATEWAY=`get_key_value ${IFCFG_FILE} IPV6_DEFAULTGW`
		if test -z "$GATEWAY" ; then
			GATEWAY="NULL"
		fi
		DNS=`get_key_value ${IFCFG_FILE} IPV6DNS`
		if test -z "$DNS" ; then
			DNS="NULL"
		fi
	fi

	${SYNONETD_TOOL} --add-gateway-info -6 -2 ${IFNAME} ${GATEWAY} ${DNS} ethernet
	${SYNONETD_TOOL} --refresh-gateway -6
}

if [ "bridge" != "`/bin/get_key_value /etc/synoinfo.conf net_topology`" ]; then
	if [ ${PORT} -eq 0 ]; then
		IFNAME="eth0"
	elif [ ${PORT} -eq 1 -a "yes" == "`/bin/get_key_value ${SECONDARY_WAN_CONF} enable`" ]; then
		IFNAME=$(/bin/get_key_value ${SECONDARY_WAN_CONF} ifname).$(/bin/get_key_value ${SECONDARY_WAN_CONF} vlan_id)
	fi

	if ! `/usr/syno/bin/synobootseq --is-booting-up > /dev/null 2>&1`; then
		# TODO: a general mapping table for switch port index to port ID
		if [ ${PORT} -eq 0 ]; then
			${SYNONET} --pppoe-start-byport 1
		elif [ ${PORT} -eq 1]; then
			${SYNONET} --pppoe-start-byport 2
		fi
	fi
	if [ "dhcp" == "`/bin/get_key_value /etc/sysconfig/network-scripts/ifcfg-${IFNAME} BOOTPROTO`" ]; then
		ifconfig $IFNAME 0.0.0.0
		if [ "running" == "`/sbin/status dhcp-client IFACE=${IFNAME} | grep -o running`" ]; then
			restart dhcp-client IFACE=$IFNAME
		else
			start dhcp-client IFACE=$IFNAME
		fi
	elif [ "static" == "`/bin/get_key_value /etc/sysconfig/network-scripts/ifcfg-${IFNAME} BOOTPROTO`" ]; then
		add_static_route_v4
	fi
	add_static_gateway_v6
	${SYNONETD_TOOL} --start-dslite ${IFNAME} &
else
	IFNAME="br0"
fi

check_and_reload_ipv6_dhcp $IFNAME
