#!/bin/sh
# Copyright (c) 2000-2014 Synology Inc. All rights reserved.

USBMODEM_INFO="/tmp/usbmodem.info"
USBMODEM_HANDLER_LOCK="/tmp/lock/lock_usbmodem_plug"
COMMON_MODULE="usbnet cdc_ether"
RNDIS_MODULE="cfg80211 rndis_host rndis_wlan"
DAEMON_NAME="synousbmodemd"
MODEM_DEBUG="/tmp/usbmodem.debug"
BIN_SYNOMODULETOOL="/usr/syno/bin/synomoduletool"

KERNEL=""
BUSPORTNUM=""
USING_MODULES=""

# return 0: true
#		1: false
# FIXME: should use upstart
check_do_start_daemon()
{
	local os_name=`/usr/syno/bin/synogetkeyvalue /etc.defaults/synoinfo.conf os_name`
	local topo_mode=`/usr/syno/bin/synogetkeyvalue /etc/synoinfo.conf net_topology`

	if [ "SRM" != "$os_name" ]; then
		return 1
	fi

	if [ "router" != "$topo_mode" -a "client" != "$topo_mode" ]; then
		return 1
	fi

	return 0
}

list_using_modules() {
	local module_names=`grep -h MODULES ${USBMODEM_INFO}* | grep -v grep | cut -d'=' -f2 | sort -u`
	local ret=""

	for module_name in ${module_names}; do
		if [ "x${module_name}" != "x" ]; then
			select_modules ${module_name}
			local filelist=${MODULE_FILES}

			for mod in ${filelist}; do
				if [ -z "$(echo ${ret} | grep ${mod})" ]; then
					ret="${mod} ${ret}"
				fi
			done
		fi
	done

	USING_MODULES=${ret}
}

get_module_list() {
	MODULE_LIST=""

	case ${SYNO_USB_DRIVER} in
		rndis_host)
			MODULE_LIST="${COMMON_MODULE} ${RNDIS_MODULE}"
			;;
		cdc-ether)
			MODULE_LIST="${COMMON_MODULE}"
			;;
		*)
			echo "unknown usbnet driver ${SYNO_USB_DRIVER}" >> ${MODEM_DEBUG}
			;;
	esac
}

insert_modules() {
	get_module_list
	local list="$MODULE_LIST"

	for mod in ${list}; do
		if [ -f /lib/modules/${mod}.ko ]; then
			echo "$BIN_SYNOMODULETOOL --insmod usb.rndis.${BUSPORTNUM} ${mod}.ko" >> ${MODEM_DEBUG}
			$BIN_SYNOMODULETOOL --insmod usb.rndis.${BUSPORTNUM} ${mod}.ko
		else
			echo "Modules ${mod} non-exists in /lib/modules/" >> ${MODEM_DEBUG}
		fi
	done
}

remove_modules() {
	get_module_list
	local list="$MODULE_LIST"
	local filelist=""
	list_using_modules

	for mod in ${list}; do
		if [ -z "$(echo ${USING_MODULES} | grep ${mod})" ]; then
			filelist="${mod} ${filelist}"
		fi
	done

	for mod in ${filelist}; do
		$BIN_SYNOMODULETOOL --rmmod usb.rndis.${BUSPORTNUM} ${mod}.ko
	done
}

plug_in_usbmodem () {
	[ ! -d /tmp/lock ] && /bin/mkdir /tmp/lock
	(flock -x 666
		generate_usb_info
		insert_modules
	) 666>${USBMODEM_HANDLER_LOCK}
}

plug_out_usbmodem () {
	[ ! -d /tmp/lock ] && /bin/mkdir /tmp/lock
	(flock -x 666
		remove_usb_info
		remove_modules
	) 666>${USBMODEM_HANDLER_LOCK}
}

set_usb_parameters () {
	local strTmp=${KERNEL//.}
	BUSPORTNUM=`echo ${strTmp//-} | cut -d ":" -f1`
}

generate_usb_info() {
	local usbmodem_info_file="${USBMODEM_INFO}${BUSPORTNUM}"

	if [ -f $usbmodem_info_file ]; then
		return
	fi

	echo "DONGLE_TYPE=${SYNO_USB_DEVICE}" >> $usbmodem_info_file
	echo "USBNET_DRIVER=${SYNO_USB_DRIVER}" >> $usbmodem_info_file

	if [ "0x12d1" == "${SYNO_USB_VENDER}" ]; then
		echo "USBNET_VENDOR=Huawei" >> $usbmodem_info_file
	elif [ "0x19d2" == "${SYNO_USB_VENDER}" ]; then
		echo "USBNET_VENDOR=ZTE" >> $usbmodem_info_file
	else
		echo "USBNET_VENDOR=NONE" >> $usbmodem_info_file
	fi
}

remove_usb_info() {
	local usbmodem_info_file="${USBMODEM_INFO}${BUSPORTNUM}"

	if [ ! -f $usbmodem_info_file ]; then
		return
	fi

	/bin/rm $usbmodem_info_file
}

# FIXME: should use upstart
start_usbmodem_daemon () {
	local hasdaemon=`ps | grep ${DAEMON_NAME} | grep -v grep`
	local hasdbus=`ps | grep dbus-daemon | grep system | grep -v grep`
	local waitCount=0;
	local maxWaitCount=60;

	check_do_start_daemon
	if [ 1 -eq $? ]; then
		return
	fi

	if [ "x${hasdaemon}" = "x" ]; then

		while [ "x${hasdbus}" = "x" ] && [ $waitCount -lt $maxWaitCount ]; do
			echo "Wait for dbus-daemon starting..." >> ${MODEM_DEBUG}
			sleep 1
			waitCount=`expr $waitCount + 1`
			hasdbus=`ps | grep dbus-daemon | grep system | grep -v grep`
		done

		if [ "x${hasdbus}" = "x" ]; then
			echo "Timeout. Skip starting ${DAEMON_NAME}" >> ${MODEM_DEBUG}
		else
			echo "dbus-daemon is running. Try to start ${DAEMON_NAME}..." >> ${MODEM_DEBUG}
			`/usr/syno/sbin/${DAEMON_NAME}`
			hasdaemon=`ps | grep ${DAEMON_NAME} | grep -v grep`

			if [ "x${hasdaemon}" = "x" ]; then
				echo "Failed to start ${DAEMON_NAME}" >> ${MODEM_DEBUG}
			else
				echo "${DAEMON_NAME} is running" >> ${MODEM_DEBUG}
			fi
		fi
	elif [ "x${hasdaemon}" != "x" ]; then
		# signal it
		echo "----------> signal it " >> ${MODEM_DEBUG}
    fi
}

# FIXME: should use upstart
terminate_usbmodem_daemon () {
	local hasdaemon=`ps | grep ${DAEMON_NAME} | grep -v grep`

	if [ "x${hasdaemon}" != "x" ]; then
		local pid=`echo ${hasdaemon} | awk '{print $1}'`
		`/bin/kill -15 ${pid}`
		local ret=$?
		echo "Try to terminate ${DAEMON_NAME}..." >> ${MODEM_DEBUG}
		if [ ${ret} -ne 0 ]; then
			echo "Failed to terminate ${DAEMON_NAME}:[${pid}, ${ret}]" >> ${MODEM_DEBUG}
		fi
		sleep 1
	elif [ "x${hasdaemon}" != "x" ]; then
		# signal it
			echo "<---------- signal it " >> ${MODEM_DEBUG}
	fi
}

set_net_parameters () {
	BUSPORTNUM=""

	if [ ! -d /sys/class/net/$KERNEL ]; then
		return
	fi

	local folderlist=`/bin/ls /sys/class/net/$KERNEL/device/driver/`

	for folder in $folderlist; do
		#/sys/class/net/usb0/device/driver/3-1:1.0/net
		local net_path="/sys/class/net/$KERNEL/device/driver/$folder/net"
		if [ ! -d $net_path ]; then
			continue
		fi

		# multiple interface attach same driver, but usually one of them has network interface
		# /sys/class/net/usb0/device/driver/3-1:1.0/net/usb0, path exist, and has interface name usb0
		# /sys/class/net/usb0/device/driver/3-1:1.1/net, path is not exist
		local net_contain=`/bin/ls $net_path`
		if [ "x$net_contain" != "x$KERNEL" ]; then
			continue
		fi

		# contain is same as $KERNEL, usb0, found!
		local strTmp=${folder//.}
		BUSPORTNUM=`echo ${strTmp//-} | cut -d ":" -f1`
		break
	done
}

generate_net_info() {
	local usbmodem_info_file="${USBMODEM_INFO}${BUSPORTNUM}"
	local strTmp=""

	if [ ! -f $usbmodem_info_file ]; then
		return
	fi

	strTmp=`/bin/get_key_value $usbmodem_info_file "DONGLE_TYPE"`
	if [ "xUSBNET" != "x$strTmp" ]; then
		return
	fi

	strTmp=`/bin/get_key_value $usbmodem_info_file "USBNET_IFNAME"`
	if [ ! -z $strTmp ]; then
		return
	fi

	echo "USBNET_IFNAME=$KERNEL" >> $usbmodem_info_file
}

set_iptables_rule() {
	iptables -A LTE_DONGLE_FORWARD -o ${KERNEL} -p tcp -m tcp --tcp-flags SYN,RST SYN -j TCPMSS --set-mss 1452
}

flush_iptables_rule() {
	iptables -F LTE_DONGLE_FORWARD
}

echo "-----------" >> ${MODEM_DEBUG}
echo "`date`" >> ${MODEM_DEBUG}
echo "action:[${ACTION}-${SUBSYSTEM}]" >> ${MODEM_DEBUG}

# when add/remove usb, $KERNEL is usb tree node path
# when add/remove net, $KERNEL is interface name, equal $INTERFACE
KERNEL=$1

case ${ACTION}-${SUBSYSTEM} in
	add-usb)
		set_usb_parameters
		plug_in_usbmodem
		start_usbmodem_daemon
		;;
	remove-usb)
		set_usb_parameters
		terminate_usbmodem_daemon
		plug_out_usbmodem
		flush_iptables_rule
		;;
	add-net)
		set_net_parameters
		generate_net_info
		set_iptables_rule
		;;
	remove-net)
		# do nothing
		;;
	*)
		echo "Usage: [add|remove]"
		;;
esac

